/*
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <math.h>

#include "ir.h"
#define IR_DEV "/dev/ir"
#define DSP_DEV "/dev/dsp"
#define EMPEG_DSP_MAGIC 'a'
#define EMPEG_DSP_BEEP _IOW(EMPEG_DSP_MAGIC, 0, int)
#define EMPEG_DSP_VOLADJ _IOW(EMPEG_DSP_MAGIC, 2, int)

main(argc,argv)
  int argc;
  char *argv[];
{
  int irrealfd;
  ssize_t rts;
  long ircode;
  int dspfd;
  int iores;
  int keepreading;
  int stdinflags;
  int doioctl = 0;
  int factorpersecond = (2 << 16);
  int real_silence = 30;
  int fake_silence = 80;
  int minvol = 3277;
  int headroom = (1 << 16);
  int args[5];
  char inarray[10];
  struct timeval tv;
  int stdinfd = 0;
  fd_set readfds;

  stdinflags = fcntl( stdinfd, F_GETFL );
  fcntl( stdinfd, F_SETFL, stdinflags | O_NONBLOCK );

  irrealfd = open(IR_DEV, O_RDONLY|O_NDELAY|O_NONBLOCK);

  if ( irrealfd == -1 ) {
    perror("Opening IR device.");
    exit(1);
  }

  dspfd = open(DSP_DEV, O_RDONLY);

  if ( dspfd == -1 ) { 
    perror("Opening DSP device.");
    close(irrealfd);
    exit(1);
  } 

  FD_ZERO( &readfds );
  FD_SET( irrealfd, &readfds );
  FD_SET( stdinfd, &readfds );


  keepreading = 1;
  while (keepreading) {
    tv.tv_sec = 5;
    tv.tv_usec = 0;
    rts = select( irrealfd + 1, &readfds, NULL, NULL, &tv );
    if (rts == 0) {
      keepreading = 0;
    } else {
      if (FD_ISSET( irrealfd, &readfds )) {
        rts = read(irrealfd, (char *) &ircode, 4);

        if (rts == 0 ) {
          perror("Finished reading from real IR device.");
          close(irrealfd);
          close(dspfd);
          exit(0);
        }

        switch (ircode) {
          case IR_TOP_BUTTON_RELEASED:
          case IR_TOP_BUTTON_PRESSED:
            keepreading = 0;
            doioctl = 0;
            break;
          case IR_RIGHT_BUTTON_RELEASED:
          case IR_RIGHT_BUTTON_PRESSED:
            keepreading = 0;
            doioctl = 1;
            minvol = 10000;
            break;
          case IR_LEFT_BUTTON_RELEASED:
          case IR_LEFT_BUTTON_PRESSED:
            keepreading = 0;
            doioctl = 1;
            minvol = 0;
            break;
          case IR_BOTTOM_BUTTON_RELEASED:
          case IR_BOTTOM_BUTTON_PRESSED:
            keepreading = 0;
            doioctl = 1;
            minvol = 3277;
            break; 
          default:
            break;
        }
      }
      if (FD_ISSET( stdinfd, &readfds )) {
        rts = read(stdinfd, inarray, 1);
        if (inarray[ 0 ] == 'q') {
          keepreading = 0;
          printf("TOLD TO QUIT\n");
        }
      }
    }
  }


  if (doioctl > 0) {
    args[ 0 ] = factorpersecond;
    args[ 1 ] = minvol;
    args[ 2 ] = headroom;
    args[ 3 ] = real_silence;
    args[ 4 ] = fake_silence;
    iores = ioctl(dspfd, EMPEG_DSP_VOLADJ, args) ;
    fprintf(stderr, "result of ioctl was: %d\n",iores);
    if (iores == -1) {
      fprintf(stderr, "errno: %d\n",errno);
    }
  } else {
    fprintf(stderr, "Not doing ioctl\n");
  }
  /* pause, then fobble up any leftover button presses */
  tv.tv_sec = 0;
  tv.tv_usec = 250000;
  rts = select( 0, NULL, NULL, NULL, &tv );
  while ( read(irrealfd, (char *) &ircode, 4) > 0 );
  close(irrealfd);
  close(dspfd);
  fcntl( stdinfd, F_SETFL, stdinflags );
  exit(0);
}
